@@ -46,10 +46,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#ifdef WIN32
-# include <io.h>
-# include <windows.h>
-#endif /* WIN32 */
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif
@@ -64,6 +60,7 @@
 #include "playlist.h"
 #include "strfctns.h"
 #include "util.h"
+#include "xalloc.h"
 
 #define STREAM_DONE	0
 #define STREAM_CONT	1
@@ -77,6 +74,7 @@ extern char		*__progname;
 char			*__progname;
 #endif /* HAVE___PROGNAME */
 
+int			 nFlag;
 int			 qFlag;
 int			 vFlag;
 int			 metadataFromProgram;
@@ -86,27 +84,31 @@ playlist_t		*playlist = NULL;
 int			 playlistMode = 0;
 
 #ifdef HAVE_SIGNALS
-const int		 ezstream_signals[] = { SIGHUP, SIGUSR1, SIGUSR2 };
+const int		 ezstream_signals[] = {
+	SIGTERM, SIGINT, SIGHUP, SIGUSR1, SIGUSR2
+};
 
 volatile sig_atomic_t	 rereadPlaylist = 0;
 volatile sig_atomic_t	 rereadPlaylist_notify = 0;
 volatile sig_atomic_t	 skipTrack = 0;
 volatile sig_atomic_t	 queryMetadata = 0;
+volatile sig_atomic_t	 quit = 0;
 #else
 int			 rereadPlaylist = 0;
 int			 rereadPlaylist_notify = 0;
 int			 skipTrack = 0;
 int			 queryMetadata = 0;
+int			 quit = 0;
 #endif /* HAVE_SIGNALS */
 
 typedef struct tag_ID3Tag {
-	char tag[3];
-	char trackName[30];
-	char artistName[30];
-	char albumName[30];
-	char year[3];
-	char comment[30];
-	char genre;
+	char	tag[3];
+	char	trackName[30];
+	char	artistName[30];
+	char	albumName[30];
+	char	year[3];
+	char	comment[30];
+	char	genre;
 } ID3Tag;
 
 int		urlParse(const char *, char **, int *, char **);
@@ -116,17 +118,18 @@ char *		buildCommandString(const char *, const char *, metadata_t *);
 char *		getMetadataString(const char *, metadata_t *);
 metadata_t *	getMetadata(const char *);
 int		setMetadata(shout_t *, metadata_t *, char **);
-FILE *		openResource(shout_t *, const char *, int *, char **, int *,
-			     int *);
+FILE *		openResource(shout_t *, const char *, int *, metadata_t **,
+			     int *, int *);
 int		reconnectServer(shout_t *, int);
 const char *	getTimeString(int);
 int		sendStream(shout_t *, FILE *, const char *, int, const char *,
-			   void *);
+			   struct timeval *);
 int		streamFile(shout_t *, const char *);
 int		streamPlaylist(shout_t *, const char *);
 char *		getProgname(const char *);
 void		usage(void);
 void		usageHelp(void);
+int		ez_shutdown(int);
 
 #ifdef HAVE_SIGNALS
 void	sig_handler(int);
@@ -135,6 +138,10 @@ void
 sig_handler(int sig)
 {
 	switch (sig) {
+	case SIGTERM:
+	case SIGINT:
+		quit = 1;
+		break;
 	case SIGHUP:
 		rereadPlaylist = 1;
 		rereadPlaylist_notify = 1;
@@ -249,12 +256,12 @@ buildCommandString(const char *extension, const char *fileName,
 		return (NULL);
 	}
 	newDecoderLen = strlen(decoder) + strlen(fileName) + 1;
-	newDecoder = xcalloc(1, newDecoderLen);
+	newDecoder = xcalloc(newDecoderLen, sizeof(char));
 	replaceString(decoder, newDecoder, newDecoderLen, TRACK_PLACEHOLDER,
 		      fileName);
 	if (strstr(decoder, ARTIST_PLACEHOLDER) != NULL) {
 		size_t tmpLen = strlen(newDecoder) + strlen(metadata_get_artist(mdata)) + 1;
-		char *tmpStr = xcalloc(1, tmpLen);
+		char *tmpStr = xcalloc(tmpLen, sizeof(char));
 		replaceString(newDecoder, tmpStr, tmpLen, ARTIST_PLACEHOLDER,
 			      metadata_get_artist(mdata));
 		xfree(newDecoder);
@@ -262,7 +269,7 @@ buildCommandString(const char *extension, const char *fileName,
 	}
 	if (strstr(decoder, TITLE_PLACEHOLDER) != NULL) {
 		size_t tmpLen = strlen(newDecoder) + strlen(metadata_get_title(mdata)) + 1;
-		char *tmpStr = xcalloc(1, tmpLen);
+		char *tmpStr = xcalloc(tmpLen, sizeof(char));
 		replaceString(newDecoder, tmpStr, tmpLen, TITLE_PLACEHOLDER,
 			      metadata_get_title(mdata));
 		xfree(newDecoder);
@@ -282,7 +289,7 @@ buildCommandString(const char *extension, const char *fileName,
 		if (metadataFromProgram && pezConfig->metadataFormat != NULL) {
 			char *mdataString = getMetadataString(pezConfig->metadataFormat, mdata);
 			size_t tmpLen = strlen(newDecoder) + strlen(mdataString) + 1;
-			char *tmpStr = xcalloc(1, tmpLen);
+			char *tmpStr = xcalloc(tmpLen, sizeof(char));
 			replaceString(newDecoder, tmpStr, tmpLen,
 				      METADATA_PLACEHOLDER, mdataString);
 			xfree(newDecoder);
@@ -291,14 +298,14 @@ buildCommandString(const char *extension, const char *fileName,
 		} else {
 			if (!metadataFromProgram && strstr(decoder, TITLE_PLACEHOLDER) != NULL) {
 				size_t tmpLen = strlen(newDecoder) + 1;
-				char *tmpStr = xcalloc(1, tmpLen);
+				char *tmpStr = xcalloc(tmpLen, sizeof(char));
 				replaceString(newDecoder, tmpStr, tmpLen,
 					      METADATA_PLACEHOLDER, "");
 				xfree(newDecoder);
 				newDecoder = tmpStr;
 			} else {
 				size_t tmpLen = strlen(newDecoder) + strlen(metadata_get_string(mdata)) + 1;
-				char *tmpStr = xcalloc(1, tmpLen);
+				char *tmpStr = xcalloc(tmpLen, sizeof(char));
 				replaceString(newDecoder, tmpStr, tmpLen,
 					      METADATA_PLACEHOLDER,
 					      metadata_get_string(mdata));
@@ -316,7 +323,7 @@ buildCommandString(const char *extension, const char *fileName,
 			       (strcmp(pezConfig->format, THEORA_FORMAT) != 0) ? " (unsupported) " : " ",
 			       pezConfig->format);
 		commandStringLen = strlen(newDecoder) + 1;
-		commandString = xcalloc(1, commandStringLen);
+		commandString = xcalloc(commandStringLen, sizeof(char));
 		strlcpy(commandString, newDecoder, commandStringLen);
 		xfree(decoder);
 		xfree(encoder);
@@ -325,12 +332,12 @@ buildCommandString(const char *extension, const char *fileName,
 	}
 
 	newEncoderLen = strlen(encoder) + strlen(metadata_get_artist(mdata)) + 1;
-	newEncoder = xcalloc(1, newEncoderLen);
+	newEncoder = xcalloc(newEncoderLen, sizeof(char));
 	replaceString(encoder, newEncoder, newEncoderLen, ARTIST_PLACEHOLDER,
 		      metadata_get_artist(mdata));
 	if (strstr(encoder, TITLE_PLACEHOLDER) != NULL) {
 		size_t tmpLen = strlen(newEncoder) + strlen(metadata_get_title(mdata)) + 1;
-		char *tmpStr = xcalloc(1, tmpLen);
+		char *tmpStr = xcalloc(tmpLen, sizeof(char));
 		replaceString(newEncoder, tmpStr, tmpLen, TITLE_PLACEHOLDER,
 			      metadata_get_title(mdata));
 		xfree(newEncoder);
@@ -340,7 +347,7 @@ buildCommandString(const char *extension, const char *fileName,
 		if (metadataFromProgram && pezConfig->metadataFormat != NULL) {
 			char *mdataString = getMetadataString(pezConfig->metadataFormat, mdata);
 			size_t tmpLen = strlen(newEncoder) + strlen(mdataString) + 1;
-			char *tmpStr = xcalloc(1, tmpLen);
+			char *tmpStr = xcalloc(tmpLen, sizeof(char));
 			replaceString(newEncoder, tmpStr, tmpLen,
 				      METADATA_PLACEHOLDER, mdataString);
 			xfree(newEncoder);
@@ -349,14 +356,14 @@ buildCommandString(const char *extension, const char *fileName,
 		} else {
 			if (!metadataFromProgram && strstr(encoder, TITLE_PLACEHOLDER) != NULL) {
 				size_t tmpLen = strlen(newEncoder) + 1;
-				char *tmpStr = xcalloc(1, tmpLen);
+				char *tmpStr = xcalloc(tmpLen, sizeof(char));
 				replaceString(newEncoder, tmpStr, tmpLen,
 					      METADATA_PLACEHOLDER, "");
 				xfree(newEncoder);
 				newEncoder = tmpStr;
 			} else {
 				size_t tmpLen = strlen(newEncoder) + strlen(metadata_get_string(mdata)) + 1;
-				char *tmpStr = xcalloc(1, tmpLen);
+				char *tmpStr = xcalloc(tmpLen, sizeof(char));
 				replaceString(newEncoder, tmpStr, tmpLen,
 					      METADATA_PLACEHOLDER,
 					      metadata_get_string(mdata));
@@ -368,7 +375,7 @@ buildCommandString(const char *extension, const char *fileName,
 
 	commandStringLen = strlen(newDecoder) + strlen(" | ") +
 		strlen(newEncoder) + 1;
-	commandString = xcalloc(1, commandStringLen);
+	commandString = xcalloc(commandStringLen, sizeof(char));
 	snprintf(commandString, commandStringLen, "%s | %s", newDecoder,
 		 newEncoder);
 
@@ -384,7 +391,7 @@ char *
 getMetadataString(const char *format, metadata_t *mdata)
 {
 	char	*tmp, *str;
-	size_t	 siz;
+	size_t	 len;
 
 	if (mdata == NULL) {
 		printf("%s: getMetadataString(): Internal error: NULL metadata_t\n",
@@ -398,33 +405,33 @@ getMetadataString(const char *format, metadata_t *mdata)
 	str = xstrdup(format);
 
 	if (strstr(format, ARTIST_PLACEHOLDER) != NULL) {
-		siz = strlen(str) + strlen(metadata_get_artist(mdata)) + 1;
-		tmp = xcalloc(1, siz);
-		replaceString(str, tmp, siz, ARTIST_PLACEHOLDER,
+		len = strlen(str) + strlen(metadata_get_artist(mdata)) + 1;
+		tmp = xcalloc(len, sizeof(char));
+		replaceString(str, tmp, len, ARTIST_PLACEHOLDER,
 			      metadata_get_artist(mdata));
 		xfree(str);
 		str = tmp;
 	}
 	if (strstr(format, TITLE_PLACEHOLDER) != NULL) {
-		siz = strlen(str) + strlen(metadata_get_title(mdata)) + 1;
-		tmp = xcalloc(1, siz);
-		replaceString(str, tmp, siz, TITLE_PLACEHOLDER,
+		len = strlen(str) + strlen(metadata_get_title(mdata)) + 1;
+		tmp = xcalloc(len, sizeof(char));
+		replaceString(str, tmp, len, TITLE_PLACEHOLDER,
 			      metadata_get_title(mdata));
 		xfree(str);
 		str = tmp;
 	}
 	if (strstr(format, STRING_PLACEHOLDER) != NULL) {
-		siz = strlen(str) + strlen(metadata_get_string(mdata)) + 1;
-		tmp = xcalloc(1, siz);
-		replaceString(str, tmp, siz, STRING_PLACEHOLDER,
+		len = strlen(str) + strlen(metadata_get_string(mdata)) + 1;
+		tmp = xcalloc(len, sizeof(char));
+		replaceString(str, tmp, len, STRING_PLACEHOLDER,
 			      metadata_get_string(mdata));
 		xfree(str);
 		str = tmp;
 	}
 	if (strstr(format, TRACK_PLACEHOLDER) != NULL) {
-		siz = strlen(str) + strlen(metadata_get_filename(mdata)) + 1;
-		tmp = xcalloc(1, siz);
-		replaceString(str, tmp, siz, TRACK_PLACEHOLDER,
+		len = strlen(str) + strlen(metadata_get_filename(mdata)) + 1;
+		tmp = xcalloc(len, sizeof(char));
+		replaceString(str, tmp, len, TRACK_PLACEHOLDER,
 			      metadata_get_filename(mdata));
 		xfree(str);
 		str = tmp;
@@ -439,7 +446,7 @@ getMetadata(const char *fileName)
 	metadata_t	*mdata;
 
 	if (metadataFromProgram) {
-		if ((mdata = metadata_program(fileName)) == NULL)
+		if ((mdata = metadata_program(fileName, nFlag)) == NULL)
 			return (NULL);
 
 		if (!metadata_program_update(mdata, METADATA_ALL)) {
@@ -447,7 +454,7 @@ getMetadata(const char *fileName)
 			return (NULL);
 		}
 	} else {
-		if ((mdata = metadata_file(fileName)) == NULL)
+		if ((mdata = metadata_file(fileName, nFlag)) == NULL)
 			return (NULL);
 
 		if (!metadata_file_update(mdata)) {
@@ -464,6 +471,7 @@ setMetadata(shout_t *shout, metadata_t *mdata, char **mdata_copy)
 {
 	shout_metadata_t	*shout_mdata = NULL;
 	char			*songInfo;
+	const char		*artist, *title;
 	int			 ret = SHOUTERR_SUCCESS;
 
 	if (shout == NULL) {
@@ -481,27 +489,61 @@ setMetadata(shout_t *shout, metadata_t *mdata, char **mdata_copy)
 		exit(1);
 	}
 
-	if ((songInfo = getMetadataString(pezConfig->metadataFormat, mdata)) == NULL) {
-		if (strlen(metadata_get_artist(mdata)) == 0 &&
-		    strlen(metadata_get_title(mdata)) == 0)
-			songInfo = xstrdup(metadata_get_string(mdata));
-		else
-			songInfo = metadata_assemble_string(mdata);
-	}
+	artist = metadata_get_artist(mdata);
+	title = metadata_get_title(mdata);
 
-	if (shout_metadata_add(shout_mdata, "song", songInfo) != SHOUTERR_SUCCESS) {
+	/*
+	 * We can do this, because we know how libshout works. This adds
+	 * "charset=UTF-8" to the HTTP metadata update request and has the
+	 * desired effect of letting newer-than-2.3.1 versions of Icecast know
+	 * which encoding we're using.
+	 */
+	if (shout_metadata_add(shout_mdata, "charset", "UTF-8") != SHOUTERR_SUCCESS) {
 		/* Assume SHOUTERR_MALLOC */
 		printf("%s: shout_metadata_add(): %s\n", __progname,
 		       strerror(ENOMEM));
 		exit(1);
 	}
+
+	if ((songInfo = getMetadataString(pezConfig->metadataFormat, mdata)) == NULL) {
+		if (artist[0] == '\0' && title[0] == '\0')
+			songInfo = xstrdup(metadata_get_string(mdata));
+		else
+			songInfo = metadata_assemble_string(mdata);
+		if (artist[0] != '\0' && title[0] != '\0') {
+			if (shout_metadata_add(shout_mdata, "artist", artist) != SHOUTERR_SUCCESS) {
+				printf("%s: shout_metadata_add(): %s\n", __progname,
+				       strerror(ENOMEM));
+				exit(1);
+			}
+			if (shout_metadata_add(shout_mdata, "title", title) != SHOUTERR_SUCCESS) {
+				printf("%s: shout_metadata_add(): %s\n", __progname,
+				       strerror(ENOMEM));
+				exit(1);
+			}
+		} else {
+			if (shout_metadata_add(shout_mdata, "song", songInfo) != SHOUTERR_SUCCESS) {
+				printf("%s: shout_metadata_add(): %s\n", __progname,
+				       strerror(ENOMEM));
+				exit(1);
+			}
+		}
+	} else if (shout_metadata_add(shout_mdata, "song", songInfo) != SHOUTERR_SUCCESS) {
+		printf("%s: shout_metadata_add(): %s\n", __progname,
+		       strerror(ENOMEM));
+		exit(1);
+	}
+
 	if ((ret = shout_set_metadata(shout, shout_mdata)) != SHOUTERR_SUCCESS)
 		printf("%s: shout_set_metadata(): %s\n",
 		       __progname, shout_get_error(shout));
+
 	shout_metadata_free(shout_mdata);
-	if (ret == SHOUTERR_SUCCESS &&
-	    mdata_copy != NULL && *mdata_copy == NULL)
-		*mdata_copy = xstrdup(songInfo);
+
+	if (ret == SHOUTERR_SUCCESS) {
+		if (mdata_copy != NULL && *mdata_copy == NULL)
+			*mdata_copy = xstrdup(songInfo);
+	}
 
 	xfree(songInfo);
 	return (ret);
@@ -509,7 +551,7 @@ setMetadata(shout_t *shout, metadata_t *mdata, char **mdata_copy)
 
 FILE *
 openResource(shout_t *shout, const char *fileName, int *popenFlag,
-	     char **metaCopy, int *isStdin, int *songLen)
+	     metadata_t **mdata_p, int *isStdin, int *songLen)
 {
 	FILE		*filep = NULL;
 	char		 extension[25];
@@ -521,11 +563,14 @@ openResource(shout_t *shout, const char *fileName, int *popenFlag,
 		if (metadataFromProgram) {
 			if ((mdata = getMetadata(pezConfig->metadataProgram)) == NULL)
 				return (NULL);
-			if (setMetadata(shout, mdata, metaCopy) != SHOUTERR_SUCCESS) {
+			if (setMetadata(shout, mdata, NULL) != SHOUTERR_SUCCESS) {
 				metadata_free(&mdata);
 				return (NULL);
 			}
-			metadata_free(&mdata);
+			if (mdata_p != NULL)
+				*mdata_p = mdata;
+			else
+				metadata_free(&mdata);
 		}
 
 		if (vFlag)
@@ -563,8 +608,6 @@ openResource(shout_t *shout, const char *fileName, int *popenFlag,
 		if ((mdata = getMetadata(fileName)) == NULL)
 			return (NULL);
 	}
-	if (metaCopy != NULL)
-		*metaCopy = metadata_assemble_string(mdata);
 	if (songLen != NULL)
 		*songLen = metadata_get_length(mdata);
 
@@ -573,7 +616,10 @@ openResource(shout_t *shout, const char *fileName, int *popenFlag,
 		int	stderr_fd = -1;
 
 		pCommandString = buildCommandString(extension, fileName, mdata);
-		metadata_free(&mdata);
+		if (mdata_p != NULL)
+			*mdata_p = mdata;
+		else
+			metadata_free(&mdata);
 		if (vFlag > 1)
 			printf("%s: Running command `%s`\n", __progname,
 			       pCommandString);
@@ -620,7 +666,10 @@ openResource(shout_t *shout, const char *fileName, int *popenFlag,
 		return (filep);
 	}
 
-	metadata_free(&mdata);
+	if (mdata_p != NULL)
+		*mdata_p = mdata;
+	else
+		metadata_free(&mdata);
 
 	if ((filep = fopen(fileName, "rb")) == NULL)
 		printf("%s: %s: %s\n", __progname, fileName,
@@ -664,7 +713,10 @@ reconnectServer(shout_t *shout, int closeConn)
 
 		printf("%s: Waiting 5s for %s to come back ...\n",
 		       __progname, pezConfig->URL);
-		sleep(5);
+		if (quit)
+			return (0);
+		else
+			sleep(5);
 	};
 
 	printf("%s: Giving up\n", __progname);
@@ -692,14 +744,13 @@ getTimeString(int seconds)
 
 int
 sendStream(shout_t *shout, FILE *filepstream, const char *fileName,
-	   int isStdin, const char *songLenStr, void *tv)
+	   int isStdin, const char *songLenStr, struct timeval *tv)
 {
 	unsigned char	 buff[4096];
 	size_t		 read, total, oldTotal;
 	int		 ret;
-#ifdef HAVE_GETTIMEOFDAY
 	double		 kbps = -1.0;
-	struct timeval	 timeStamp, *startTime = (struct timeval *)tv;
+	struct timeval	 timeStamp, *startTime = tv;
 
 	if (startTime == NULL) {
 		printf("%s: sendStream(): Internal error: startTime is NULL\n",
@@ -709,7 +760,6 @@ sendStream(shout_t *shout, FILE *filepstream, const char *fileName,
 
 	timeStamp.tv_sec = startTime->tv_sec;
 	timeStamp.tv_usec = startTime->tv_usec;
-#endif /* HAVE_GETTIMEOFDAY */
 
 	total = oldTotal = 0;
 	ret = STREAM_DONE;
@@ -733,6 +783,8 @@ sendStream(shout_t *shout, FILE *filepstream, const char *fileName,
 			}
 		}
 
+		if (quit)
+			break;
 		if (rereadPlaylist_notify) {
 			rereadPlaylist_notify = 0;
 			if (!pezConfig->fileNameIsProgram)
@@ -754,10 +806,8 @@ sendStream(shout_t *shout, FILE *filepstream, const char *fileName,
 
 		total += read;
 		if (qFlag && vFlag) {
-#ifdef HAVE_GETTIMEOFDAY
 			struct timeval	tv;
 			double		oldTime, newTime;
-#endif /* HAVE_GETTIMEOFDAY */
 
 			if (!isStdin && playlistMode) {
 				if (pezConfig->fileNameIsProgram) {
@@ -771,10 +821,9 @@ sendStream(shout_t *shout, FILE *filepstream, const char *fileName,
 					       playlist_get_num_items(playlist));
 			}
 
-#ifdef HAVE_GETTIMEOFDAY
 			oldTime = (double)timeStamp.tv_sec
 				+ (double)timeStamp.tv_usec / 1000000.0;
-			gettimeofday(&tv, NULL);
+			ez_gettimeofday((void *)&tv);
 			newTime = (double)tv.tv_sec
 				+ (double)tv.tv_usec / 1000000.0;
 			if (songLenStr == NULL)
@@ -794,7 +843,6 @@ sendStream(shout_t *shout, FILE *filepstream, const char *fileName,
 				printf("                 ");
 			else
 				printf("  [%8.2f kbps]", kbps);
-#endif /* HAVE_GETTIMEOFDAY */
 
 			printf("  \r");
 			fflush(stdout);
@@ -805,7 +853,7 @@ sendStream(shout_t *shout, FILE *filepstream, const char *fileName,
 			clearerr(filepstream);
 			ret = STREAM_CONT;
 		} else
-			printf("%s: streamFile(): Error while reading '%s': %s\n",
+			printf("%s: sendStream(): Error while reading '%s': %s\n",
 			       __progname, fileName, strerror(errno));
 	}
 
@@ -817,39 +865,46 @@ streamFile(shout_t *shout, const char *fileName)
 {
 	FILE		*filepstream = NULL;
 	int		 popenFlag = 0;
-	char		*metaData = NULL, *songLenStr = NULL;
+	char		*songLenStr = NULL;
 	int		 isStdin = 0;
 	int		 ret, retval = 0, songLen;
-#ifdef HAVE_GETTIMEOFDAY
+	metadata_t	*mdata;
 	struct timeval	 startTime;
-#endif
 
 	if ((filepstream = openResource(shout, fileName, &popenFlag,
-					&metaData, &isStdin, &songLen))
+					&mdata, &isStdin, &songLen))
 	    == NULL) {
 		return (retval);
 	}
 
-	if (metaData != NULL) {
+	if (mdata != NULL) {
+		char	*tmp, *metaData;
+
+		tmp = metadata_assemble_string(mdata);
+		metaData = UTF8toCHAR(tmp, ICONV_REPLACE);
+		xfree(tmp);
 		printf("%s: Streaming ``%s''", __progname, metaData);
 		if (vFlag)
 			printf(" (file: %s)\n", fileName);
 		else
 			printf("\n");
 		xfree(metaData);
+
+		/* MP3 streams are special, so set the metadata explicitly: */
+		if (strcmp(pezConfig->format, MP3_FORMAT) == 0)
+			setMetadata(shout, mdata, NULL);
+
+		metadata_free(&mdata);
 	}
 
-#ifdef HAVE_GETTIMEOFDAY
 	if (songLen >= 0)
 		songLenStr = xstrdup(getTimeString(songLen));
-	gettimeofday(&startTime, NULL);
+	ez_gettimeofday((void *)&startTime);
 	do {
 		ret = sendStream(shout, filepstream, fileName, isStdin,
-				 songLenStr, (void *)&startTime);
-#else
-	do {
-		ret = sendStream(shout, filepstream, fileName, isStdin, NULL, NULL);
-#endif
+				 songLenStr, &startTime);
+		if (quit)
+			break;
 		if (ret != STREAM_DONE) {
 			if ((skipTrack && rereadPlaylist) ||
 			    (skipTrack && queryMetadata)) {
@@ -872,22 +927,22 @@ streamFile(shout_t *shout, const char *fileName)
 				queryMetadata = 0;
 				if (metadataFromProgram) {
 					char		*mdataStr = NULL;
-					metadata_t	*mdata;
+					metadata_t	*prog_mdata;
 
 					if (vFlag > 1)
 						printf("%s: Querying '%s' for fresh metadata\n",
 						       __progname, pezConfig->metadataProgram);
-					if ((mdata = getMetadata(pezConfig->metadataProgram)) == NULL) {
+					if ((prog_mdata = getMetadata(pezConfig->metadataProgram)) == NULL) {
 						retval = 0;
 						ret = STREAM_DONE;
 						continue;
 					}
-					if (setMetadata(shout, mdata, &mdataStr) != SHOUTERR_SUCCESS) {
+					if (setMetadata(shout, prog_mdata, &mdataStr) != SHOUTERR_SUCCESS) {
 						retval = 0;
 						ret = STREAM_DONE;
 						continue;
 					}
-					metadata_free(&mdata);
+					metadata_free(&prog_mdata);
 					printf("%s: New metadata: ``%s''\n",
 					       __progname, mdataStr);
 					xfree(mdataStr);
@@ -941,6 +996,8 @@ streamPlaylist(shout_t *shout, const char *fileName)
 		strlcpy(lastSong, song, sizeof(lastSong));
 		if (!streamFile(shout, song))
 			return (0);
+		if (quit)
+			break;
 		if (rereadPlaylist) {
 			rereadPlaylist = rereadPlaylist_notify = 0;
 			if (pezConfig->fileNameIsProgram)
@@ -961,12 +1018,16 @@ streamPlaylist(shout_t *shout, const char *fileName)
 	return (1);
 }
 
-/* Borrowed from OpenNTPd-portable's compat-openbsd/bsd-misc.c */
+/*
+ * Borrowed from OpenNTPd-portable's compat-openbsd/bsd-misc.c.
+ * Does not use xalloc on purpose, as the 9 bytes of memory that don't get
+ * cleaned up in the end really don't matter.
+ */
 char *
 getProgname(const char *argv0)
 {
 #ifdef HAVE___PROGNAME
-	return (xstrdup(__progname));
+	return (strdup(__progname));
 #else
 	char	*p;
 
@@ -978,22 +1039,34 @@ getProgname(const char *argv0)
 	else
 		p++;
 
-	return (xstrdup(p));
+	return (strdup(p));
 #endif /* HAVE___PROGNAME */
 }
 
+int
+ez_shutdown(int exitval)
+{
+	shout_shutdown();
+	playlist_shutdown();
+	freeConfig(pezConfig);
+	xalloc_shutdown();
+
+	return (exitval);
+}
+
 void
 usage(void)
 {
-	printf("usage: %s [-hqVv] [-c configfile]\n", __progname);
+	printf("usage: %s [-hnqVv] -c configfile\n", __progname);
 }
 
 void
 usageHelp(void)
 {
 	printf("\n");
-	printf("  -c configfile  use XML configuration in configfile\n");
+	printf("  -c configfile  use XML configuration in configfile (mandatory)\n");
 	printf("  -h             display this additional help and exit\n");
+	printf("  -n             normalize metadata strings\n");
 	printf("  -q             suppress STDERR output from external en-/decoders\n");
 	printf("  -V             print the version number and exit\n");
 	printf("  -v             verbose output (use twice for more effect)\n");
@@ -1017,38 +1090,50 @@ main(int argc, char *argv[])
 	unsigned int	 i;
 #endif
 
+#ifdef XALLOC_DEBUG
+	xalloc_initialize_debug(2, NULL);
+#else
+	xalloc_initialize();
+#endif /* XALLOC_DEBUG */
+	playlist_init();
+	shout_init();
+
 	__progname = getProgname(argv[0]);
 	pezConfig = getEZConfig();
 
+	nFlag = 0;
 	qFlag = 0;
 	vFlag = 0;
 
-	while ((c = getopt(argc, argv, "c:hqVv")) != -1) {
+	while ((c = getopt(argc, argv, "c:hnqVv")) != -1) {
 		switch (c) {
 		case 'c':
 			if (configFile != NULL) {
 				printf("Error: multiple -c arguments given\n");
 				usage();
-				return (2);
+				return (ez_shutdown(2));
 			}
 			configFile = xstrdup(optarg);
 			break;
 		case 'h':
 			usage();
 			usageHelp();
-			return (0);
+			return (ez_shutdown(0));
+		case 'n':
+			nFlag = 1;
+			break;
 		case 'q':
 			qFlag = 1;
 			break;
 		case 'V':
 			printf("%s\n", PACKAGE_STRING);
-			return (0);
+			return (ez_shutdown(0));
 		case 'v':
 			vFlag++;
 			break;
 		case '?':
 			usage();
-			return (2);
+			return (ez_shutdown(2));
 		default:
 			break;
 		}
@@ -1059,7 +1144,7 @@ main(int argc, char *argv[])
 	if (configFile == NULL) {
 		printf("You must supply a config file with the -c argument.\n");
 		usage();
-		return (2);
+		return (ez_shutdown(2));
 	} else {
 		/*
 		 * Attempt to open configFile here for a more meaningful error
@@ -1072,7 +1157,7 @@ main(int argc, char *argv[])
 		if (stat(configFile, &st) == -1) {
 			printf("%s: %s\n", configFile, strerror(errno));
 			usage();
-			return (2);
+			return (ez_shutdown(2));
 		}
 		if (vFlag && (st.st_mode & (S_IRGRP | S_IROTH)))
 			printf("%s: Warning: %s is group and/or world readable\n",
@@ -1080,7 +1165,7 @@ main(int argc, char *argv[])
 		if (st.st_mode & (S_IWGRP | S_IWOTH)) {
 			printf("%s: Error: %s is group and/or world writeable\n",
 			       __progname, configFile);
-			return (2);
+			return (ez_shutdown(2));
 		}
 #else
 		FILE		 *tmp;
@@ -1088,43 +1173,40 @@ main(int argc, char *argv[])
 		if ((tmp = fopen(configFile, "r")) == NULL) {
 			printf("%s: %s\n", configFile, strerror(errno));
 			usage();
-			return (2);
+			return (ez_shutdown(2));
 		}
 		fclose(tmp);
 #endif /* HAVE_STAT */
 	}
 
 	if (!parseConfig(configFile))
-		return (2);
-
-	shout_init();
-	playlist_init();
+		return (ez_shutdown(2));
 
 	if (pezConfig->URL == NULL) {
 		printf("%s: Error: Missing <url>\n", configFile);
-		return (2);
+		return (ez_shutdown(2));
 	}
 	if (!urlParse(pezConfig->URL, &host, &port, &mount)) {
 		printf("Must be of the form ``http://server:port/mountpoint''\n");
-		return (2);
+		return (ez_shutdown(2));
 	}
 	if (strlen(host) == 0) {
 		printf("%s: Error: Invalid <url>: Missing server:\n", configFile);
 		printf("Must be of the form ``http://server:port/mountpoint''\n");
-		return (2);
+		return (ez_shutdown(2));
 	}
 	if (strlen(mount) == 0) {
 		printf("%s: Error: Invalid <url>: Missing mountpoint:\n", configFile);
 		printf("Must be of the form ``http://server:port/mountpoint''\n");
-		return (2);
+		return (ez_shutdown(2));
 	}
 	if (pezConfig->password == NULL) {
 		printf("%s: Error: Missing <sourcepassword>\n", configFile);
-		return (2);
+		return (ez_shutdown(2));
 	}
 	if (pezConfig->fileName == NULL) {
 		printf("%s: Error: Missing <filename>\n", configFile);
-		return (2);
+		return (ez_shutdown(2));
 	}
 	if (pezConfig->format == NULL) {
 		printf("%s: Warning: Missing <format>:\n", configFile);
@@ -1133,120 +1215,8 @@ main(int argc, char *argv[])
 
 	xfree(configFile);
 
-	if ((shout = shout_new()) == NULL) {
-		printf("%s: shout_new(): %s", __progname, strerror(ENOMEM));
-		return (1);
-	}
-
-	if (shout_set_host(shout, host) != SHOUTERR_SUCCESS) {
-		printf("%s: shout_set_host(): %s\n", __progname,
-			shout_get_error(shout));
-		return (1);
-	}
-	if (shout_set_protocol(shout, SHOUT_PROTOCOL_HTTP) != SHOUTERR_SUCCESS) {
-		printf("%s: shout_set_protocol(): %s\n", __progname,
-			shout_get_error(shout));
-		return (1);
-	}
-	if (shout_set_port(shout, port) != SHOUTERR_SUCCESS) {
-		printf("%s: shout_set_port: %s\n", __progname,
-			shout_get_error(shout));
-		return (1);
-	}
-	if (shout_set_password(shout, pezConfig->password) != SHOUTERR_SUCCESS) {
-		printf("%s: shout_set_password(): %s\n", __progname,
-			shout_get_error(shout));
-		return (1);
-	}
-	if (shout_set_mount(shout, mount) != SHOUTERR_SUCCESS) {
-		printf("%s: shout_set_mount(): %s\n", __progname,
-			shout_get_error(shout));
-		return (1);
-	}
-	if (shout_set_user(shout, "source") != SHOUTERR_SUCCESS) {
-		printf("%s: shout_set_user(): %s\n", __progname,
-			shout_get_error(shout));
-		return (1);
-	}
-
-	if (!strcmp(pezConfig->format, MP3_FORMAT)) {
-		if (shout_set_format(shout, SHOUT_FORMAT_MP3) != SHOUTERR_SUCCESS) {
-			printf("%s: shout_set_format(MP3): %s\n",
-			       __progname, shout_get_error(shout));
-			return (1);
-		}
-	}
-	if (!strcmp(pezConfig->format, VORBIS_FORMAT) ||
-	    !strcmp(pezConfig->format, THEORA_FORMAT)) {
-		if (shout_set_format(shout, SHOUT_FORMAT_OGG) != SHOUTERR_SUCCESS) {
-			printf("%s: shout_set_format(OGG): %s\n",
-			       __progname, shout_get_error(shout));
-			return (1);
-		}
-	}
-
-	if (pezConfig->serverName) {
-		if (shout_set_name(shout, pezConfig->serverName) != SHOUTERR_SUCCESS) {
-			printf("%s: shout_set_name(): %s\n",
-			       __progname, shout_get_error(shout));
-			return (1);
-		}
-	}
-	if (pezConfig->serverURL) {
-		if (shout_set_url(shout, pezConfig->serverURL) != SHOUTERR_SUCCESS) {
-			printf("%s: shout_set_url(): %s\n",
-			       __progname, shout_get_error(shout));
-			return (1);
-		}
-	}
-	if (pezConfig->serverGenre) {
-		if (shout_set_genre(shout, pezConfig->serverGenre) != SHOUTERR_SUCCESS) {
-			printf("%s: shout_set_genre(): %s\n",
-			       __progname, shout_get_error(shout));
-			return (1);
-		}
-	}
-	if (pezConfig->serverDescription) {
-		if (shout_set_description(shout, pezConfig->serverDescription) != SHOUTERR_SUCCESS) {
-			printf("%s: shout_set_description(): %s\n",
-			       __progname, shout_get_error(shout));
-			return (1);
-		}
-	}
-	if (pezConfig->serverBitrate) {
-		if (shout_set_audio_info(shout, SHOUT_AI_BITRATE, pezConfig->serverBitrate) != SHOUTERR_SUCCESS) {
-			printf("%s: shout_set_audio_info(AI_BITRATE): %s\n",
-			       __progname, shout_get_error(shout));
-			return (1);
-		}
-	}
-	if (pezConfig->serverChannels) {
-		if (shout_set_audio_info(shout, SHOUT_AI_CHANNELS, pezConfig->serverChannels) != SHOUTERR_SUCCESS) {
-			printf("%s: shout_set_audio_info(AI_CHANNELS): %s\n",
-			       __progname, shout_get_error(shout));
-			return (1);
-		}
-	}
-	if (pezConfig->serverSamplerate) {
-		if (shout_set_audio_info(shout, SHOUT_AI_SAMPLERATE, pezConfig->serverSamplerate) != SHOUTERR_SUCCESS) {
-			printf("%s: shout_set_audio_info(AI_SAMPLERATE): %s\n",
-			       __progname, shout_get_error(shout));
-			return (1);
-		}
-	}
-	if (pezConfig->serverQuality) {
-		if (shout_set_audio_info(shout, SHOUT_AI_QUALITY, pezConfig->serverQuality) != SHOUTERR_SUCCESS) {
-			printf("%s: shout_set_audio_info(AI_QUALITY): %s\n",
-			       __progname, shout_get_error(shout));
-			return (1);
-		}
-	}
-
-	if (shout_set_public(shout, pezConfig->serverPublic) != SHOUTERR_SUCCESS) {
-		printf("%s: shout_set_public(): %s\n",
-		       __progname, shout_get_error(shout));
-		return (1);
-	}
+	if ((shout = stream_setup(host, port, mount)) == NULL)
+		return (ez_shutdown(1));
 
 	if (pezConfig->metadataProgram != NULL)
 		metadataFromProgram = 1;
@@ -1263,28 +1233,23 @@ main(int argc, char *argv[])
 		if (sigaction(ezstream_signals[i], &act, NULL) == -1) {
 			printf("%s: sigaction(): %s\n",
 			       __progname, strerror(errno));
-			return (1);
+			return (ez_shutdown(1));
 		}
 	}
 #endif /* HAVE_SIGNALS */
 
 	if (shout_open(shout) == SHOUTERR_SUCCESS) {
 		int	ret;
-		char	*tmpFileName, *p;
 
 		printf("%s: Connected to http://%s:%d%s\n", __progname,
 		       host, port, mount);
 
-		tmpFileName = xstrdup(pezConfig->fileName);
-		for (p = tmpFileName; *p != '\0'; p++)
-			*p = tolower((int)*p);
 		if (pezConfig->fileNameIsProgram ||
-		    strrcmp(tmpFileName, ".m3u") == 0 ||
-		    strrcmp(tmpFileName, ".txt") == 0)
+		    strrcasecmp(pezConfig->fileName, ".m3u") == 0 ||
+		    strrcasecmp(pezConfig->fileName, ".txt") == 0)
 			playlistMode = 1;
 		else
 			playlistMode = 0;
-		xfree(tmpFileName);
 
 		if (vFlag && pezConfig->fileNameIsProgram)
 			printf("%s: Using program '%s' to get filenames for streaming\n",
@@ -1298,25 +1263,26 @@ main(int argc, char *argv[])
 			} else {
 				ret = streamFile(shout, pezConfig->fileName);
 			}
+			if (quit)
+				break;
 			if (pezConfig->streamOnce)
 				break;
 		} while (ret);
+
+		shout_close(shout);
 	} else
 		printf("%s: Connection to http://%s:%d%s failed: %s\n", __progname,
 		       host, port, mount, shout_get_error(shout));
 
+	if (quit)
+		printf("\r%s: SIGINT or SIGTERM received\n", __progname);
+
 	if (vFlag)
 		printf("%s: Exiting ...\n", __progname);
 
-	shout_close(shout);
-
-	playlist_free(&playlist);
-	playlist_shutdown();
-
-	shout_shutdown();
-
 	xfree(host);
 	xfree(mount);
+	playlist_free(&playlist);
 
-	return 0;
+	return (ez_shutdown(0));
 }
