@@ -33,7 +33,7 @@
 
 #include "compat.h"
 #include "playlist.h"
-#include "util.h"
+#include "xalloc.h"
 
 extern char	*__progname;
 
@@ -82,7 +82,7 @@ playlist_add(playlist_t *pl, const char *entry)
 	}
 
 	if (pl->size / sizeof(char *) <= num) {
-		unsigned long	i;
+		size_t	i;
 
 		pl->list = xrealloc(pl->list, 2, pl->size);
 		pl->size = 2 * pl->size;
@@ -171,10 +171,8 @@ playlist_read(const char *filename)
 			continue;
 
 		/* Trim any trailing newlines and carriage returns. */
-		if (buf[strlen(buf) - 1] == '\n')
-			buf[strlen(buf) - 1] = '\0';
-		if (buf[0] != '\0' && buf[strlen(buf) - 1] == '\r')
-			buf[strlen(buf) - 1] = '\0';
+		buf[strcspn(buf, "\n")] = '\0';
+		buf[strcspn(buf, "\r")] = '\0';
 
 		/* Skip lines that are empty after the trimming. */
 		if (buf[0] == '\0')
@@ -242,36 +240,39 @@ playlist_program(const char *filename)
 }
 
 void
-playlist_free(playlist_t **pl)
+playlist_free(playlist_t **pl_p)
 {
-	size_t		i;
-	playlist_t	*tmp;
+	size_t		 i;
+	playlist_t	*pl;
 
-	if (pl == NULL || *pl == NULL)
+	if (pl_p == NULL || (pl = *pl_p) == NULL)
 		return;
 
-	tmp = *pl;
+	if (pl->filename != NULL) {
+		xfree(pl->filename);
+		pl->filename = NULL;
+	}
 
-	if (tmp->filename != NULL)
-		xfree(tmp->filename);
-
-	if (tmp->list != NULL) {
-		if (tmp->size > 0) {
-			for (i = 0; i < tmp->size / sizeof(char *); i++) {
-				if (tmp->list[i] != NULL)
-					xfree(tmp->list[i]);
-				else
+	if (pl->list != NULL) {
+		if (pl->size > 0) {
+			for (i = 0; i < pl->size / sizeof(char *); i++) {
+				if (pl->list[i] != NULL) {
+					xfree(pl->list[i]);
+					pl->list[i] = NULL;
+				} else
 					break;
 			}
 		}
 
-		xfree(tmp->list);
+		xfree(pl->list);
 	}
 
-	if (tmp->prog_track != NULL)
-		xfree(tmp->prog_track);
+	if (pl->prog_track != NULL) {
+		xfree(pl->prog_track);
+		pl->prog_track = NULL;
+	}
 
-	xfree(*pl);
+	xfree(*pl_p);
 }
 
 const char *
@@ -437,8 +438,8 @@ playlist_reread(playlist_t **plist)
 void
 playlist_shuffle(playlist_t *pl)
 {
-	unsigned long	 d, i, range;
-	char		*temp;
+	size_t	 d, i, range;
+	char	*temp;
 
 	if (pl == NULL) {
 		printf("%s: playlist_shuffle(): Internal error: NULL argument\n",
@@ -518,10 +519,8 @@ playlist_run_program(playlist_t *pl)
 		return (NULL);
 	}
 
-	if (buf[0] != '\0' && buf[strlen(buf) - 1] == '\n')
-		buf[strlen(buf) - 1] = '\0';
-	if (buf[0] != '\0' && buf[strlen(buf) - 1] == '\r')
-		buf[strlen(buf) - 1] = '\0';
+	buf[strcspn(buf, "\n")] = '\0';
+	buf[strcspn(buf, "\r")] = '\0';
 	if (buf[0] == '\0') {
 		printf("%s: Empty line received from program '%s'\n",
 		       __progname, pl->filename);
